# SALMON: Scalable Ab-initio Light-Matter simulator for Optics and Nanoscience

SALMON is an open-source software based on first-principles time-dependent density functional theory
to describe optical responses and electron dynamics in matters induced by light electromagnetic fields.

SALMON has been developed in such a way that it runs optimally in the following supercomputer platforms:

- K-computer
- Fujitsu FX100 supercomputer system
- Linux PC Cluster with x86-64 CPU
- Linux PC Cluster with Intel Xeon Phi processor

For more information, please visit our website.

http://salmon-tddft.jp/wiki/

## License

SALMON is available under Apache License version 2.0.

    Copyright 2017 SALMON developers
    
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
  
       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

## Files

This package is composed of the following files:

- `configure.py` The python script to build SALMON. It uses CMake for the build process, and is the default choice for the build process.

- `CMakeList.txt` CMake script that is used in `configure.py`.

- `cmakefiles` (directory) CMake related files.

- `examples` (directory) A few sample calculations using SALMON are included. Detailed explanations of these examples are described in the web site, <http://salmon-tddft.jp/wiki/Exercises>.

- `LICENCE` License document of SALMON.

- `makefiles` (directory) GNU Makefiles to build SALMON.

- `manual` (directory) A list of namalist variables that is usable in input files.

- `README.md` This file.

- `src` (directory) Source code files of SALMON.

- `testsuites` (directory) Files that are used in the automatic test of SALMON. It can also be used to examine if the build process is successful in the user's environment.

- `tools` (directory) Some tools and utilities related to SALMON.

## Install and Run

### Prerequisites

In this guide, it is assumed that readers have a basic knowledge of Unix and its command line operations. For the installation of SALMON, following packages are required.

- Fortran90/C compiler. SALMON assumes users have one of the following compilers:

  - GCC (Gnu Compiler Collection)
  - Intel Fortran/C Compiler
  - Fujitsu Compiler (at FX100 / K-Computer)

- One of the following library packages for linear algebra:

  - BLAS/LAPACK
  - Intel Math Kernel Library (MKL)
  - Fujitsu Scientific Subroutine Library 2 (SSL-II)

- Build tools.

  - CMake

If you use other compilers, you may need to change build scripts (CMake). See [1]. If no numerical library is installed on your computer system, you may need to install BLAS/LAPACK by yourself. See [link][2].

For the installation of SALMON, we adopt the CMake tools as the first option. If there were any problems to use CMake tools in your environment, you may use the GNU make tools. See [link][2].

### Download

The newest version of SALMON can be downloaded from [3]. To extract files from the downloaded file `*salmon-<VERSION>.tar.gz*`, type the following command in the command-line,

`$ tar 窶奴xvf ./salmon-<VERSION>.tar.gz`

After the extraction, the following directories will be created.

`SALMON`<br>
`|- src Source codes`<br>
`|- example Samples`<br>
`|- makefiles GNU Makefiles for building`<br>
`|- cmakefiles CMake related files`<br>
`|- makefiles GNU Makefiles for building`

### Build

To compile SALMON to create executable the binary files, we adopt to use CMake tools as the first option. In case you fail to build SALMON using CMake in your environment, we may use Gnu Make. See [link][4].

#### Checking CMake availability

First, examine whether CMake is usable in your environment or not. Type the following in Unix command-line:

`$ cmake --version`

If CMake is not installed in your system, an error message such as `cmake: command not found` will appear. If CMake is installed on your system, the version number will be shown. To build SALMON, CMake of version 3.0.2 or later is required. If you confirm that CMake of version 3.0.2 or later is installed in your system, proceed to [link][4]. However, we realize that old versions of CMake are installed in many systems. If CMake is not installed or CMake of older versions is installed in your system, you need to install the new version by yourself. It is a simple procedure and explained below.

#### Installation of CMake

[CMake][5] is a cross-platform build tool. The simplest way to make CMake usable in your environment is to get the binary distribution of CMake from the [download page][6]. The file name of the binary distribution will be `cmake-<VERSION>-<PLATFORM>.tar.gz`). In standard Unix environme

[1]: http://salmon-tddft.jp/wiki/Install_and_Run#Additional_options_in_configure.py_script
[2]: http://salmon-tddft.jp/wiki/Troubleshooting_of_the_Installation_Process
[3]: http://salmon-tddft.jp/wiki/Download
[4]: http://salmon-tddft.jp/wiki/Install_and_Run#Additional_options_in_configure.py_script#Build_using_GNU_Makefile
[5]: https://cmake.org/
[6]: https://cmake.org/download/
